package com.yunfeiyun.agmp.fms.mapper;

import com.yunfeiyun.agmp.fms.domain.FmsCrop;
import com.yunfeiyun.agmp.fms.domain.resvo.FmsBlockResVo;
import com.yunfeiyun.agmp.fms.domain.resvo.FmsCropResVo;

import java.util.List;

/**
 * 作物Mapper接口
 * 
 * @author 杨晓辉
 * @date 2023-05-16
 */
public interface FmsCropMapper 
{
    /**
     * 查询作物
     * 
     * @param cropId 作物主键
     * @return 作物
     */
    public FmsCrop selectFmsCropByCropId(String cropId);

    /**
     * 根据类型查询作物名称集合
     *
     * @param cropMaintype 类型
     * @return 作物
     */
    public List<FmsCrop> selectFmsCropNameByCropMaintype(String cropMaintype);
    public List<FmsBlockResVo> selectFmsCropNameByBlockId(String[] blockIds);


    /**
     * 查询作物列表
     * 
     * @param fmsCrop 作物
     * @return 作物集合
     */
    public List<FmsCropResVo> selectFmsCropList(FmsCrop fmsCrop);

    /**
     * 新增作物
     * 
     * @param fmsCrop 作物
     * @return 结果
     */
    public int insertFmsCrop(FmsCrop fmsCrop);

    /**
     * 修改作物
     * 
     * @param fmsCrop 作物
     * @return 结果
     */
    public int updateFmsCrop(FmsCrop fmsCrop);

    /**
     * 删除作物
     * 
     * @param cropId 作物主键
     * @return 结果
     */
    public int deleteFmsCropByCropId(String cropId);

    /**
     * 批量删除作物
     * 
     * @param cropIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteFmsCropByCropIds(String[] cropIds);
}
